<?php
/*
 * Plugin Name: NoFollow
 * Author: Arun S. Sekher [Physioblast, Arun]
 * Author URI: http://e107themes.net/
 * Requirement: e107 v0.7.xx
 *
 * Copyright (C) 2011 Arun S. Sekher (http://e107themes.net)
 * Released under the terms and conditions of the
 * GNU General Public License (http://www.gnu.org/licenses/gpl.txt)
 *
 * $HeadURL: https://nofollow.svn.sourceforge.net/svnroot/nofollow/trunk/0.3/nofollow.php $ 
 * $Revision: 25 $
 * $Date: 2011-05-08 17:48:21 +0400 (Sun, 08 May 2011) $
 * $Author: arunshekher $
 *
*/
if (!defined('e107_INIT')) { exit; }

 /**
  * Class e_nofollow
  * @category e107 plugins
  * @package nofollow e107 plugin
  * @author Arun S. Sekher
  * @copyright (C) 2011 Arun S. Sekher <arun@e107themes.net>
  * @license GNU General Public License
  * @link <http://e107themes.net>
  * @version 0.3 beta 
  */
  class e_nofollow{

  	const VERSION = '0.3';
	private
		// @property bool nofollow status
		$nf_active = false,	
		// @property bool nofollow onpost
		$nf_post = false,	
		// @property bool log nofollow on post
		$nf_logpost = true,	
		// @property arr flags for various contexts
		$nf_context = array(),	
		// @property arr omited pages 
		$nf_omitpages = array(),	
		// @property arr omited domains 
		$nf_omitdomains = array();	

	/**
     * Construct the e_nofollow Object
	 * Grabs nofollow prefs, does some checks, assign values etc.
	 * 
	 * @global $sysprefs e107 prefs class instance
	 * @global arr $nofollow_prefs = plugin prefs
	 * @var str $current_url = current page with server query string
	 * @var str $site = e107 site which runs this plugin
	 * @access public
     */
    public function __construct(){	
	  global $sysprefs, $nofollow_prefs;
	    if(!is_object($sysprefs)){ $sysprefs = new prefs; }
	    $nofollow_prefs = $sysprefs->getArray('nofollow_prefs');    	
		
		//omit on admin pages (adopted from linkwords)
		if ((strpos(e_SELF, ADMINDIR) !== false) || 
			(strpos(e_PAGE, "admin_") !== false)) return;    	
		
		// omit on admin opted pages (adopted from linkwords)
		$current_url = e_SELF.(e_QUERY ? "?".e_QUERY : '');
		$this->nf_omitpages = explode('|', substr($nofollow_prefs['nf_omit_pages'], 2));
		  foreach ($this->nf_omitpages as $p){
			if ( $p = trim($p) ){
			  if( substr($p, -1) == '!' ){
				$p = substr($p, 0, -1);
				if(substr($current_url, strlen($p)*-1) == $p) return;				
			  }else{
				if(strpos($current_url, $p) !== FALSE) return;				
			  }
			}
		  }		 		 		
		// set nofollow status
		$this->nf_active = $nofollow_prefs['nf_active'];
		
		// set omit domains
        $this->nf_omitdomains = explode('|', $nofollow_prefs['nf_omit_domains']);
        $site = str_replace(array('http://', 'www.', 'http://www.'), '', SITEURLBASE);
		array_unshift($this->nf_omitdomains, $site);
		$this->nf_omitdomains =  array_unique($this->nf_omitdomains);
		//print_r($this->nf_omitdomains);
		
		// set context
		$this->nf_context = $nofollow_prefs['nf_context_visibility'];
			
	}
	
	/**
	 * Logs calls to makeNoFollow() func onpost, ie: when user submits
	 * Uses the e107 'generic' table to store log data
	 * 
	 * @var int $nf_count = makeNoFollow() call count
	 * @var arr $fldsvals = feilds and corres. values for the db insert.
	 * @access private
	 */
	private function log_nf_post(){      
	  $nf_count = 1;
	  if(!is_object($sql)){ $sql = new db; }
	  if($sql->db_Select("generic", "gen_intdata", "gen_type = 'nf_log'")){
		$rs = $sql->db_Fetch(MYSQL_ASSOC);
		$nf_count += $rs['gen_intdata'];
	    $sql->db_Update("generic", "gen_intdata = {$nf_count} WHERE gen_type = 'nf_log'");
        /*Debug*/ 
		//echo '<script type="text/javascript">alert("'.$nf_count.'");</script>';
	  }else{
		$fldsvals = array("gen_type" => "nf_log", "gen_intdata" => $nf_count);
		$sql->db_Insert("generic", $fldsvals);
	  }
	}
	
	/**
	 * Checks if the href attribute value of the <a tag fragment is in
	 * the user preferred omited domains list, retuns true if it does.
	 * makes a very basic check for internal links as well
	 * 
	 * @param str $str
	 * @return bool $found
	 * @access private
	 */
	private function domain_check($str){
	  $pattern = '/<a\s+(?:[^"\'>]+|"[^"]*"|\'[^\']*\')*href=("[^"]+"|\'[^\']+\'|[^<>\s]+)/i';		
	  preg_match($pattern, $str, $match);
	  $url = $match[1];
	  if(strpos($url, 'http') === false && strstr($url, '://')=== false) return true;		
	  $found = false; 
	  foreach ($this->nf_omitdomains as $domain) { 
		if (strpos($url, $domain) !== false) { 
			$found = true; 
			break; 
		} 
	  } 
	  return ($found);		
	}
		
	/**
	 * Adds rel="nofollow" attribute to html anchor tags if not present.
	 * If already have an rel attr. but no nofollow, appends nofollow to the end. 
	 * inserts rel="nofollow" for everything else.
	 * 
	 * @param str $anchor - string with opening anchor tag that is passed in
	 * @return str - The modified opening anchor tag string
	 * @access private
	 */
	private function makeNoFollow($anchor){
		if(strpos($anchor, 'nofollow')){ // have nofollow
			return $anchor;
		}
		elseif(strpos($anchor, 'rel')){	// have rel but no nofollow	
			$pattern = "/rel=([\"'])([^\\1]+?)\\1/";
			$replace = "rel=\\1\\2 nofollow\\1";			
			return preg_replace($pattern, $replace, $anchor);
		}
		else{ // everything else
			$pattern = "/<a /";
			$replace = "<a rel=\"nofollow\" ";
			return preg_replace($pattern, $replace, $anchor);			               
		}		
	}	
	
	/**
	 * Scans through $text for anchor tags after spliting up by html tags
	 * adopted from linkwords plugin.
	 * 
	 * @param str $text - The anchor string that will be altered
	 * @param str $opts['context'] - default context
	 * @param bool $logflag - switch to log the makenofollow on post
	 * @return str $nftext - The modified $text
	 * @access public
	 */	
	public function nofollow(&$text, $area = 'olddefault', $logflag = false){
		//var_dump($area); 
		//FIXME: omit contexts not working as expected
		if(!$this->nf_active /*|| !array_key_exists($area, $this->nf_context) 
		|| !$this->nf_context[$area]*/) return $text;		
		$nftext = '';
		$fragments = preg_split('#(<.*?>)#mis', $text, -1, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE );
		//print_r($fragments);		
		foreach($fragments as $fragment) {			
			if (strpos($fragment, '<a') !== FALSE && !strpos($fragment, '</a')){// probably an opening anchor tag				
				if($this->domain_check($fragment)){ // omited domains check
					$nftext .= $fragment;
				}else{
					$nftext .= $this->makeNoFollow($fragment);					
					if($this->nf_logpost && $logflag) $this->log_nf_post(); //loging it
				}			
			}else{ // it's some text in between	or closing a tag		
				$nftext .= $fragment;				
			}
		}
		return $nftext;
	  }	
  }

  $nofollow = new e_nofollow;
  
  //Debug
  //print_r($nofollow->nofollow_prefs);
  //$user_defines = get_defined_constants(true);
  //print_r( $user_defines['user'] );
 
?>
